﻿using System;
using System.Windows.Forms;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WinForms;
using System.Collections.ObjectModel;

namespace _8870
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            LayerOverlay layerOverlay = new LayerOverlay();
            
            EllipseShape ellipseShape = new EllipseShape(new PointShape(0, 0), 10);
            RingShape ring = ellipseShape.GetConvexHull();
            ring.Vertices.RemoveAt(ring.Vertices.Count - 1);
            LineShape curveLine = new LineShape(ring.Vertices);

            //LineShape curvedLine = (LineShape)curveLine.GetLineOnALine(new PointShape(-5, -5), new PointShape(10, 0));
            
            // line 1
            LineShape curvedLine = GetNewLine(curveLine, new PointShape(-5, -5), new PointShape(10, 0));

            // line 2
            //LineShape curvedLine = GetNewLine(curveLine, new PointShape(10, 0), new PointShape(-5, -5));

            InMemoryFeatureLayer layer = new InMemoryFeatureLayer();
            layer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColors.Black, 2, true);
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            layer.InternalFeatures.Add(new Feature(curvedLine));

            layerOverlay.Layers.Add(layer);

            winformsMap1.Overlays.Add(layerOverlay);

            winformsMap1.CurrentExtent = new RectangleShape(-20, 20, 20, -20);
            winformsMap1.Refresh();
        }

        private LineShape GetNewLine(LineShape baseline, PointShape startPoint, PointShape endPoint)
        {
            int p1 = GetClosedVertex(baseline, startPoint);
            int p2 = GetClosedVertex(baseline, endPoint);

            if (p1 <= p2)
            {
                return baseline.GetLineOnALine(startPoint, endPoint) as LineShape;
            }
            else
            {
                LineShape newLine = new LineShape();

                for (int i = p1; i < baseline.Vertices.Count; i++)
                {
                    newLine.Vertices.Add(baseline.Vertices[i]);
                }

                for (int i = 0; i <= p2; i++)
                {
                    newLine.Vertices.Add(baseline.Vertices[i]);
                }

                return newLine;
            }
        }

        private int GetClosedVertex(LineShape line, PointShape point)
        {
            double minDistance = double.MaxValue;
            int index = -1;

            for (int i = 0; i < line.Vertices.Count; i++)
            {
                Vertex v = line.Vertices[i];
                PointShape p = new PointShape(v);
                double d = p.GetDistanceTo(point, winformsMap1.MapUnit, DistanceUnit.Meter);

                if (d < minDistance)
                {
                    minDistance = d;
                    index = i;
                }
            }

            return index;
        }
        
    }  
}
